import pyglet
import json
from time import time
from random import randint

pyglet.resource.path.append("data")


pyglet.resource.reindex()
class Actor(object):
    alive = True
    pickitem = pyglet.resource.media("pick.wav", streaming=False)
    badpick = pyglet.resource.media("badpick.wav", streaming=False)
    def __init__(self, x, y, game):
        self.game = game
        self.x = x
        self.y = y
        self.speed = 60 # in pixels per second
        self.keys = None # setup by the Game class
        self.gravity = 0
        self.gravity_top = 10

    def draw(self):
        self.image.blit(self.x, self.y)

    def terrain_collision(self, x, y):
        collision = False
        for block in self.game.terrain:
            if x + self.image.width - 4  > block.x \
                and block.x + block.image.width > x + 1 \
                and y + self.image.height -5 > block.y \
                and block.y + block.image.height > y + 1 \
                and block.transpasable == False:
                collision = True
                break # collision found, leave the loop
        return collision

    def item_taken(self, x, y):

        for item in self.game.terrain:
            if x + self.image.width - 4 > item.x + 8 \
                and item.x + item.image.width - 8 > x \
                and y + self.image.height - 5 > item.y + 8 \
                and item.y + item.image.height - 8 > y + 1 \
                and isinstance(item, Item):
                if item.objeto == "clock":
                    self.game.timer.COUNTDOWN += 5
                    self.game.timer.countdown.text = "+5 seconds of life"
                    self.pickitem.play()
                elif item.objeto == self.game.gooditem:
                    self.score += 100
                    self.game.timer.countdown.text = "+100 points"
                    self.pickitem.play()
                elif self.score >= 50:
                    self.score -= 50
                    self.game.timer.countdown.text = "-50 points"
                    self.badpick.play()
                else:
                    self.badpick.play()

                self.game.score.playerscore.text = "Points: " + str(self.score)
                item.delete()
                self.game.items -= 1
                



class Player (Actor):

    animation = pyglet.resource.image("guyanim.png")
    animgrid = pyglet.image.ImageGrid (animation,1,6,32,64)
    score = 0
    jump = pyglet.resource.media("jump.wav", streaming=False)

    def __init__(self, x, y, game):
        self.game = game
        self.image = self.animgrid [0]
        self.terrain = game.terrain
        self.x = x
        self.y = y
        self.speed = 115 # in pixels per second
        self.keys = None # setup by the Game class
        self.falling = True 
        self.gravity = 0
        self.gravity_top = 40
        self.jumping = False 
        self.accel = 0
        self.elapsed = 0
        self.walk = 0

    def update(self,dt):
        # position change based on dt and speed
        change = self.speed * dt
        self.elapsed += dt

        if self.keys[pyglet.window.key.UP] \
                and not self.jumping and self.falling and self.alive:
            self.jumping = True
            self.accel = 14
            if not self.terrain_collision(self.x, self.y + change + self.accel):
                self.jump.play ()

        #Jumping and falling implementation
        if self.jumping and self.accel > 0 \
                and not self.terrain_collision(self.x, self.y + change + self.accel):
            if self.keys[pyglet.window.key.LEFT]:
                self.image = self.animgrid [4]
            elif self.keys[pyglet.window.key.RIGHT]:
                self.image = self.animgrid [2]
            self.y += change + self.accel 
            self.accel -= 1
            self.falling = False
        elif self.jumping and not self.falling:
            self.falling = True
            self.accel = 0 
        
        #gravity implementation
        elif self.falling \
                and not self.terrain_collision(self.x, self.y - change):
            if self.keys[pyglet.window.key.LEFT]:
                self.image = self.animgrid [4]
            elif self.keys[pyglet.window.key.RIGHT]:
                self.image = self.animgrid [2]
            self.y -= change - self.accel 
            self.accel -= 0.4 
       
        else:
            for block in self.terrain:
                if self.x + self.image.width -4 > block.x \
                    and block.x + block.image.width > self.x + 1 \
                    and self.y + self.image.height -5 > block.y \
                    and block.y + block.image.height > self.y + 1 \
                    and block.transpasable == False:
                    self.y = block.y + block.image.height
                    self.accel = 0
            self.jumping = False
        

        if self.keys[pyglet.window.key.LEFT] and self.alive \
                and not self.terrain_collision(self.x - change, self.y):
            self.x -= change
            if self.elapsed > 0.2 and not self.jumping:
                if self.walk == 0:
                    self.image = self.animgrid [5]
                    self.walk = 1
                    self.elapsed = 0
                elif self.walk == 1:
                    self.image = self.animgrid [4]
                    self.walk = 2
                    self.elapsed = 0
                elif self.walk == 2:
                    self.image = self.animgrid [3]
                    self.walk = 0
                    self.elapsed = 0
            
        if self.keys[pyglet.window.key.RIGHT] and self.alive \
                and not self.terrain_collision(self.x + change, self.y):
            self.x += change
            if self.elapsed > 0.2 and not self.jumping:
                if self.walk == 0:
                    self.image = self.animgrid [0]
                    self.walk = 1
                    self.elapsed = 0
                elif self.walk == 1:
                    self.image = self.animgrid [1]
                    self.walk = 2
                    self.elapsed = 0
                elif self.walk == 2:
                    self.image = self.animgrid [2]
                    self.walk = 0
                    self.elapsed = 0

        if self.keys[pyglet.window.key.X]:
            self.game.musicplayer.pause()
            self.game.game.game.go_menu()
            
        self.item_taken(self.x, self.y)



class Block(object):
    
    transpasable = True
    tileset = pyglet.resource.image("tileset.png")
    tilesetgrid = pyglet.image.ImageGrid (tileset, 1, 4, 32, 32)
    alive = True

    def __init__(self, x, y):
        self.x = x
        self.y = y
    
    def draw(self):
        self.image.blit(self.x, self.y)

class Terrain(Block):

    HEIGHT = 32
    WIDTH = 32
    transpasable = False

    def __init__(self, x, y, tipo):
        self.x = x
        self.y = y
        if tipo == 1:
            self.image = self.tilesetgrid [0]

class Item (Block):
    objecto = ""
    
    def __init__(self, x, y):
        self.x = x
        self.y = y
        
        chance = randint (1, 5)
        if  chance == 1 or chance == 4:
            self.image = self.tilesetgrid [1]
            self.objeto = "star"
        elif chance == 2:
            self.image = self.tilesetgrid [2]
            self.objeto = "clock"
        elif chance == 3 or chance == 5:
            self.image = self.tilesetgrid [3]
            self.objeto = "money"

        

    def delete (self):
        self.alive = False

class Countdown (object):
    spam = 0
    tick = pyglet.resource.media("tick.wav", streaming= False)
    def __init__(self, deadtime, game):
        self.game = game
        self.COUNTDOWN = deadtime
        self.start = time()
        self.x = Game.WIDTH // 2
        self.y = 10 
        self.countdown = pyglet.text.Label ('HURRY UP! You\'re time is running down',
                font_size=12,
                x = self.x , y = self.y,
                anchor_x='center', anchor_y='center')

    def update(self):
        timeelapsed = time () - self.start 
        if timeelapsed >= 0.5 and self.COUNTDOWN < 5 and self.spam == 0:
            self.tick.play()
            self.spam += 1
            if self.game.items <= 10:
                chance = randint (1, 5)
                if chance == 1:
                    self.game.randomitems (self.game.level, "data")
        if timeelapsed >= 1 and self.COUNTDOWN > 0 :
            self.game.changegood()
            if self.game.items <= 15:
                chance = randint (1, 15)
                if chance == 1:
                    self.game.randomitems (self.game.level, "data")
            self.spam = 0
            self.tick.play()
            self.start = time()
            self.COUNTDOWN -= 1
            self.countdown.text = 'Time to end: ' + str(self.COUNTDOWN)
        if self.COUNTDOWN == 0:
            self.game.player.alive = False
            self.countdown.text = 'GAME OVER push X for Main Menu'

    def draw(self):
        self.countdown.draw()
    
class Scoreboard (object):
    tileset = pyglet.resource.image("itembox.png")
    tilesetgrid = pyglet.image.ImageGrid (tileset, 1, 2, 64, 64)

    def __init__(self, game):
        self.game = game
        self.playerscore = pyglet.text.Label ('Points: 0', font_size=12,
                x = 5, y = 10, anchor_x='left', anchor_y='center')
        if self.game.gooditem == "star":
            self.image = self.tilesetgrid[0]
        elif self.game.gooditem == "money":
            self.image = self.tilesetgrid[1]

    def update (self):
        if self.game.gooditem == "star":
            self.image = self.tilesetgrid[0]
        elif self.game.gooditem == "money":
            self.image = self.tilesetgrid[1]
        

    def draw (self):
        self.playerscore.draw()
        self.image.blit(Game.WIDTH/2 - 32, 22)


class Game (pyglet.window.Window):
    """Main Game Window"""

    #Window Size (Don't modify)
    WIDTH = 1024
    HEIGHT = 768

    def __init__(self):
        """Initialize the game"""
        #track key state
        self.keys = pyglet.window.key.KeyStateHandler()
        self.push_handlers(self.keys)

        self.go_menu()

        ## Init base class
        super(Game, self).__init__(width=self.WIDTH, height=self.HEIGHT)

        ## Enable openGL for .png Alpha channel
        pyglet.gl.glEnable(pyglet.gl.GL_BLEND)
        pyglet.gl.glBlendFunc (pyglet.gl.GL_SRC_ALPHA, 
                pyglet.gl.GL_ONE_MINUS_SRC_ALPHA)

        # set the update interval
        pyglet.clock.schedule_interval(self.update, 1/60.0) # every 60Hz

    def go_menu (self):
        self.scene = MenuScene (self)


    def update (self, dt):
        """Update the game state"""
        self.scene.update(dt)


    def on_draw (self):
        """Draw the window contents"""
        self.clear()
        self.scene.on_draw() 


class MenuScene (object):
    
    def __init__(self, window):
        self.music = pyglet.resource.media("menu.ogg") 
        self.game = window
        self.image = pyglet.resource.image("menu.png")
        self.musicplayer = self.music.play ()

    def update (self, dt):
        if self.game.keys[pyglet.window.key.S]:
            self.game.scene = GameScene (self, self.game.keys)
            self.musicplayer.pause ()
        elif self.game.keys[pyglet.window.key.I]:
            self.image = pyglet.resource.image("instructions.png")
        elif self.game.keys[pyglet.window.key.Q]:
            pyglet.app.exit ()
        else:
            self.image = pyglet.resource.image("menu.png")

    def on_draw (self):
        self.image.blit(0,0)

class GameScene (object):
    """Game Scene"""

    #Window Size (Don't modify)
    WIDTH = 1024
    HEIGHT = 768
    items = 0

    refill = pyglet.resource.media("refill.wav", streaming= False)
    def __init__(self, window, keys):
        """Initialize the game"""
        self.music = pyglet.resource.media("game.ogg")
        self.gooditem = "star"
        self.game = window
        self.musicplayer = self.music.play ()
        # set the update interval
        # pyglet.clock.schedule_interval(self.update, 1/60.0) # every 60Hz

        ## Init game state
        self.terrain = []
        # We use json for create maps in files. The maps are located in /data
        with pyglet.resource.file("maps.json","r") as fd:
            level = json.load(fd)

        #load the map and init the player
        self.load_map (level, "data")

        #track key state (player)
        self.player.keys = keys 
        
        # create the Countdown and the Scoreboard
        self.timer = Countdown(10, self)
        self.score = Scoreboard (self)

    def update (self, dt):
        """Update the game state"""
        self.player.update(dt)
        self.timer.update()

        self.terrain = [item for item in self.terrain if item.alive]
        self.score.update()

    def on_draw (self):
        """Draw the window contents"""
        self.player.draw()
        for i in self.terrain:
            i.draw()
        self.timer.draw()
        self.score.draw()

    def changegood(self):
        if randint (1,2) == 1:
            self.gooditem = "star"
        else:
            self.gooditem = "money"


    def start (self, level,levelmap):
        start = False
        self.level = level
        leveldata = level[levelmap]
        level_h = level["height"]
        level_w = level["width"]
        spacex = 0
        spacey = self.HEIGHT
        for y in range(level_h):
            spacey -= Terrain.HEIGHT  
            for x in range(level_w):
                if leveldata[x+y*level_w] == 9:
                #if 9 is found we insert the player here
                    chance = randint (1, 5)
                    if chance == 1 and start == False:
                        self.player = Player(x+spacex, y+spacey, self)
                        start = True
                spacex += Terrain.WIDTH -1 
            spacex = 0
        return start

    def randomitems (self, level, levelmap):
        self.level = level
        leveldata = level[levelmap]
        level_h = level["height"]
        level_w = level["width"]
        spacex = 0
        spacey = self.HEIGHT
        for y in range(level_h):
            spacey -= Terrain.HEIGHT  
            for x in range(level_w):
                if leveldata[x+y*level_w] == 2:
                    chance = randint (1, 2)
                    if chance == 1:
                        self.terrain.append (Item(x+spacex, y+spacey))
                        self.items += 1
                spacex += Terrain.WIDTH -1 
            spacex = 0
        self.refill.play()


    def load_map(self,level, levelmap):
        self.level = level
        begin = False
        leveldata = level[levelmap]
        level_h = level["height"]
        level_w = level["width"]
        spacex = 0
        spacey = self.HEIGHT
        for y in range(level_h):
            spacey -= Terrain.HEIGHT  
            for x in range(level_w):
                if leveldata[x+y*level_w] == 1:
                # if 1 is found we insert a block tile
                    self.terrain.append (Terrain(x+spacex, y+spacey, 1))
                elif leveldata[x+y*level_w] == 2:
                # If 2 put a random item for collect
                    chance = randint (1, 2)
                    if chance == 1:
                        self.terrain.append (Item(x+spacex, y+spacey))
                        self.items += 1
                    # Add more objects randomized
                spacex += Terrain.WIDTH -1 
            spacex = 0

        # Insert the player position randomly
        while not begin:
            begin = self.start (level, levelmap)



def main():
    """ your app starts here
    """
    game = Game()
    pyglet.app.run()

